import nltk
from nltk.stem import WordNetLemmatizer
lemmatizer = WordNetLemmatizer()
import pickle
import numpy as np
import pyttsx3
from keras.models import load_model
import os
import json
import random

class ChatBot:
    def __init__(self, model_path=None, words_path=None, classes_path=None, data_path=None):
        if model_path and words_path and classes_path and data_path:
            self.model = self.load_model(model_path)
            self.words, self.classes = self.load_words_classes(words_path, classes_path)
            self.intents = self.load_intents(data_path)
        else:
            # Load default model and data files
            self.model = self.load_model_default()
            self.words, self.classes = self.load_words_classes_default()
            self.intents = self.load_intents_default()

    def load_model_default(self):
        current_dir = os.path.dirname(__file__)
        model_path = os.path.join(current_dir, 'model', 'chatbot_model2.h5')
        return load_model(model_path)

    def load_model(self, model_path):
        return load_model(model_path)

    def load_intents_default(self):
        current_dir = os.path.dirname(__file__)
        intents_path = os.path.join(current_dir, 'data', 'default_intents.json')
        with open(intents_path) as file:
            return json.load(file)

    def load_intents(self, data_path):
        with open(data_path) as file:
            return json.load(file)

    def load_words_classes_default(self):
        current_dir = os.path.dirname(__file__)
        words_path = os.path.join(current_dir, 'model', 'words.pkl')
        classes_path = os.path.join(current_dir, 'model', 'classes.pkl')
        with open(words_path, 'rb') as words_file:
            words = pickle.load(words_file)
        with open(classes_path, 'rb') as classes_file:
            classes = pickle.load(classes_file)
        return words, classes

    def load_words_classes(self, words_path, classes_path):
        with open(words_path, 'rb') as words_file:
            words = pickle.load(words_file)
        with open(classes_path, 'rb') as classes_file:
            classes = pickle.load(classes_file)
        return words, classes

    def clean_up_sentence(self, sentence):
        sentence_words = nltk.word_tokenize(sentence)
        sentence_words = [lemmatizer.lemmatize(word.lower()) for word in sentence_words]
        return sentence_words

    def bow(self, sentence, show_details=True):
        sentence_words = self.clean_up_sentence(sentence)
        bag = [0] * len(self.words)
        for s in sentence_words:
            for i, w in enumerate(self.words):
                if w == s:
                    bag[i] = 1
                    if show_details:
                        print("Found in bag: %s" % w)
        return np.array(bag)

    def predict_class(self, sentence):
        p = self.bow(sentence, show_details=False)
        res = self.model.predict(np.array([p]))[0]
        ERROR_THRESHOLD = 0.25
        results = [[i, r] for i, r in enumerate(res) if r > ERROR_THRESHOLD]
        results.sort(key=lambda x: x[1], reverse=True)
        return_list = []
        for r in results:
            return_list.append({"intent": self.classes[r[0]], "probability": str(r[1])})
        return return_list

    def get_response(self, sentence):
        ints = self.predict_class(sentence)
        tag = ints[0]['intent']
        list_of_intents = self.intents['intents']
        for i in list_of_intents:
            if i['tag'] == tag:
                return random.choice(i['responses'])
        return "Sorry, I don't understand that."

    def take_command(self):
        return input("Enter your sentence: ")

    def speak(self, response):
        engine = pyttsx3.init()
        voices = engine.getProperty('voices')
        engine.setProperty('voice', voices[0].id)
        engine.say(response)
        engine.runAndWait()



import os
import json
import nltk
import numpy as np
from keras.models import Sequential
from keras.layers import Dense, Dropout
from keras.optimizers import Adam
from nltk.stem import WordNetLemmatizer
import pickle
import random

class ChatBotTrainer:
    def __init__(self, mode='default'):
        current_dir = os.getcwd()  # Now points to the user's working directory

        # Ensure 'models' directory exists in the current working directory
        models_dir = os.path.join(current_dir, 'models')
        os.makedirs(models_dir, exist_ok=True)  

        self.model_path = os.path.join(models_dir, 'chatbot_model.h5')
        self.words_path = os.path.join(models_dir, 'words.pkl')
        self.classes_path = os.path.join(models_dir, 'classes.pkl')

        self.default_intents_path = os.path.join(current_dir, 'data', 'default_intents.json')

        self.lemmatizer = WordNetLemmatizer()
        self.words = []
        self.classes = []
        self.documents = []
        self.ignore_words = ['?']
        self.mode = mode

    def preprocess_data(self, intents):
        for intent in intents['intents']:
            for pattern in intent['patterns']:
                w = nltk.word_tokenize(pattern)
                self.words.extend(w)
                self.documents.append((w, intent['tag']))
                if intent['tag'] not in self.classes:
                    self.classes.append(intent['tag'])

        self.words = [self.lemmatizer.lemmatize(w.lower()) for w in self.words if w not in self.ignore_words]
        self.words = sorted(list(set(self.words)))
        self.classes = sorted(list(set(self.classes)))

        training = []
        output_empty = [0] * len(self.classes)

        for doc in self.documents:
            bag = []
            pattern_words = doc[0]
            pattern_words = [self.lemmatizer.lemmatize(word.lower()) for word in pattern_words]

            for w in self.words:
                bag.append(1) if w in pattern_words else bag.append(0)

            output_row = list(output_empty)
            output_row[self.classes.index(doc[1])] = 1
            training.append([bag, output_row])

        random.shuffle(training)

        X = np.array([entry[0] for entry in training])
        y = np.array([entry[1] for entry in training])

        return X, y

    def train_default(self, epochs=100, batch_size=5, verbose=1):
        with open(self.default_intents_path, 'r') as file:
            intents_data = json.load(file)

        return self.train(intents_data, epochs=epochs, batch_size=batch_size, verbose=verbose)

    def train_custom(self, custom_intents, epochs=100, batch_size=5, verbose=1):
        return self.train(custom_intents, epochs=epochs, batch_size=batch_size, verbose=verbose)

    def train_from_file(self, file_path, epochs=100, batch_size=5, verbose=1):
        with open(file_path, 'r') as file:
            intents_data = json.load(file)

        return self.train(intents_data, epochs=epochs, batch_size=batch_size, verbose=verbose)

    def train(self, intents_data, epochs=100, batch_size=5, verbose=1):
        X, y = self.preprocess_data(intents_data)

        model = Sequential([
            Dense(512, input_shape=(len(X[0]),), activation='relu'),
            Dropout(0.5),
            Dense(256, activation='relu'),
            Dropout(0.5),
            Dense(128, activation='relu'),
            Dropout(0.5),
            Dense(len(y[0]), activation='softmax')
        ])

        optimizer = Adam(learning_rate=0.001)
        model.compile(loss='categorical_crossentropy', optimizer=optimizer, metrics=['accuracy'])

        model.fit(X, y, epochs=epochs, batch_size=batch_size, verbose=verbose)

        # Save the model and related data
        model.save(self.model_path)
        with open(self.words_path, 'wb') as words_file:
            pickle.dump(self.words, words_file)
        with open(self.classes_path, 'wb') as classes_file:
            pickle.dump(self.classes, classes_file)

        print(f"Chatbot training complete! Epochs: {epochs}, Batch Size: {batch_size}, Verbose: {verbose}")

        return model, self.words, self.classes
    



from setuptools import setup, find_packages

setup(
    name="mlchatbotai",
    version="0.10",
    packages=find_packages(),  # Automatically find all sub-packages
    include_package_data=True,  # Ensures non-Python files (models, JSON) are included
    install_requires=[
        "numpy",
        "nltk",
        "keras",
        "tensorflow",
        "pyttsx3"
    ],
    author="Shibam Das",
    author_email="shibomdas121@gmail.com",
    description="A simple AI chatbot package",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/ShibamDas007/MLChatbotAI",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
